//
//  $Id$
//
// Copyright (c) 2003 Daniel J. Gregor, Jr.
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 
//    - Redistributions of source code must retain the above copyright
//      notice, this list of conditions and the following disclaimer.
//    - Redistributions in binary form must reproduce the above
//      copyright notice, this list of conditions and the following
//      disclaimer in the documentation and/or other materials provided
//      with the distribution.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
// FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
// COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
// INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
// BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
// ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.
//

package com.gregor.mailping;

import java.io.*;
import java.util.*;

public class MailPing {
	public static final String property_base = "mailping";

	protected Timer timer;
	protected Vector tests;
	protected PrintStream log = System.out;

	public MailPing() {
		timer = new Timer();
		tests = new Vector();
	}

	public void addTest(SMTPTest test) {
		tests.add(test);
	}

	public void run() {
		String bind_address =
		    System.getProperty(property_base + ".listener.bindaddress",
		    "*");
		SMTPListener listener = new SMTPListener(bind_address, log);
                Thread listener_thread = new Thread(listener);
                listener_thread.start();

		for (Enumeration e = tests.elements(); e.hasMoreElements(); ) {
			((SMTPTest) e.nextElement()).schedule(timer);
		}
	}

	public void setLog(PrintStream log) {
		this.log = log;
	}

	public PrintStream getLog() {
		return log;
	}

	public static void main(String argv[]) {
		MailPing mp = new MailPing();

		String str;

		long default_period = SMTPTest.default_period;
		long default_delay = SMTPTest.default_delay;
	
		try {
			loadProperties();
		} catch (IOException e) {
			e.printStackTrace();
			System.exit(1);
		}

		HashSet h = new HashSet();
		Properties p = System.getProperties();
		
		for (Enumeration e = p.propertyNames(); e.hasMoreElements();) {
			String key = (String) e.nextElement();
			String target;
			int i, j;

			if (!key.startsWith(property_base + ".target.")) {
				continue;
			}
			i = property_base.length() + ".target.".length();

			if ((j = key.substring(i).lastIndexOf(".")) == -1) {
				continue;
			}

			target = key.substring(i, i + j);
			if (target.lastIndexOf(".") != -1) {
				continue;
			}

			if (!h.contains(target)) {
				h.add(target);
			}
		}

		if (h.size() == 0) {
			System.err.println("No targets specified.  At least " +
			    "one target is required.");
			System.exit(1);
		}

		if ((str = System.getProperty(property_base + ".defaultperiod"))
		    != null) {
			try {
				default_period = Long.parseLong(str);
			} catch (NumberFormatException e) {
				System.err.println("Property " +
				    property_base + ".defaultperiod (\"" +
				    str + "\") is " +
				    "not a decimal number.");
				System.exit(1);
			}
		}

		if ((str = System.getProperty(property_base + ".defaultdelay"))
		    != null) {
			try {
				default_delay = Long.parseLong(str);
			} catch (NumberFormatException e) {
				System.err.println("Property " +
				    property_base + ".defaultdelay (\"" +
				    str + "\") is " +
				    "not a decimal number.");
				System.exit(1);
			}
		}

		if ((str = System.getProperty(property_base + ".logfile"))
		    != null) {
			try {
				mp.setLog(new PrintStream(
				    new FileOutputStream(str, true)));
			} catch (FileNotFoundException e) {
				System.err.println("Could not open logfile " +
				    "specified in property " + property_base +
				    ".logfile (\"" + str + "\")");
				e.printStackTrace();
				System.exit(1);
			}
		}

		for (Iterator it = h.iterator(); it.hasNext();) {
			String target = (String) it.next();
			String target_base = property_base + ".target." +
			    target;

			String host;
			String sender;
			Vector recipients = new Vector();
			long period = default_period;
			long delay = default_delay;
	
			int i, j;
	
			if ((host = System.getProperty(target_base + ".host"))
			    == null) {
				System.err.println("Property " + target_base +
				    ".host must be provided.");
				System.exit(1);
			}

			if ((sender = System.getProperty(target_base +
			    ".sender")) == null) {
				System.err.println("Property " + target_base +
				    ".sender must be provided.");
				System.exit(1);
			}

			if ((str = System.getProperty(target_base +
			    ".recipients")) == null) {
				System.err.println("Property " + target_base +
				    ".recipients must be provided.");
				System.exit(1);
			}

			for (i = 0; (j = str.substring(i).indexOf(",")) != -1;
			    i = j + 1) {
				recipients.add(str.substring(i, j));
			}
			recipients.add(str.substring(i));

			if ((str = System.getProperty(target_base + ".period"))
			    != null) {
				try {
					period = Long.parseLong(str);
				} catch (NumberFormatException e) {
					System.err.println("Property " +
					    target_base + ".period (\"" +
					    str + "\") is " +
					    "not a decimal number.");
					System.exit(1);
				}
			}
	
			if ((str = System.getProperty(target_base + ".delay"))
			    != null) {
				try {
					delay = Long.parseLong(str);
				} catch (NumberFormatException e) {
					System.err.println("Property " +
					    target_base + ".delay (\"" +
					    str + "\") is " +
					    "not a decimal number.");
					System.exit(1);
				}
			}

			mp.addTest(new SMTPTest(host, sender, recipients,
			    period, delay));
		}

		mp.run();
	}

	public static void loadProperties() throws IOException {
		Properties p = System.getProperties();
		String f;

		if ((f = System.getProperty(property_base + ".properties"))
		    == null) {
			if (new File(property_base + ".properties").exists()) {
				f = property_base + ".properties";
			} else {
				return;
			}
		}

		p.load(new FileInputStream(f));
		System.setProperties(p);
	}
}
